@@ -14,7 +14,7 @@ from kabaret.app.ui.gui.widgets.editors import editor_factory
 from kabaret.flow_contextual_dict import ContextualView, get_contextual_dict
 from kabaret.subprocess_manager.runners import Explorer
 
-from .users import Users, UserEnvironment, UserBookmarks
+from .users import Users, UserEnvironment, UserBookmarks, UserProfile
 from .film import Sequences
 from .lib import Assets
 from .runners import (
@@ -42,20 +42,14 @@ class LoginPageWidget(CustomPageWidget):
         self.error_label = QtWidgets.QLabel(self)
         self.error_label.setText("")
 
-        # Set server url to login page param value
-        self.server_url_oid = self.oid + "/admin/login_page/kitsu_server_url"
-        self.lineedit_server_url = QtWidgets.QLineEdit(
-            self.session.cmds.Flow.get_value(self.server_url_oid)
-        )
-
-        self.lineedit_username = QtWidgets.QLineEdit()
-        self.lineedit_password = QtWidgets.QLineEdit()
+        self.lineedit_kitsu_id = QtWidgets.QLineEdit()
+        self.lineedit_kitsu_password = QtWidgets.QLineEdit()
         self.button_login = QtWidgets.QPushButton("Log in")
         self.button_login.setMinimumWidth(100)
 
-        # Set password field input mode
-        self.lineedit_password.setEchoMode(QtWidgets.QLineEdit.Password)
-        self.lineedit_password.setInputMethodHints(
+        # Set kitsu_password field input mode
+        self.lineedit_kitsu_password.setEchoMode(QtWidgets.QLineEdit.Password)
+        self.lineedit_kitsu_password.setInputMethodHints(
             QtCore.Qt.ImhHiddenText
             | QtCore.Qt.ImhNoPredictiveText
             | QtCore.Qt.ImhNoAutoUppercase
@@ -63,11 +57,10 @@ class LoginPageWidget(CustomPageWidget):
 
         grid_layout = QtWidgets.QGridLayout()
         grid_layout.addWidget(self.label, 0, 0)
-        grid_layout.addWidget(self.lineedit_server_url, 1, 0, 1, 2)
-        grid_layout.addWidget(self.lineedit_username, 2, 0, 1, 2)
-        grid_layout.addWidget(self.lineedit_password, 3, 0, 1, 2)
-        grid_layout.addWidget(self.error_label, 4, 0)
-        grid_layout.addWidget(self.button_login, 4, 1)
+        grid_layout.addWidget(self.lineedit_kitsu_id, 1, 0, 1, 2)
+        grid_layout.addWidget(self.lineedit_kitsu_password, 2, 0, 1, 2)
+        grid_layout.addWidget(self.error_label, 3, 0)
+        grid_layout.addWidget(self.button_login, 3, 1)
         grid_layout.setColumnStretch(0, 1)
 
         self.setLayout(grid_layout)
@@ -75,25 +68,13 @@ class LoginPageWidget(CustomPageWidget):
         self.button_login.clicked.connect(self.on_button_login_clicked)
 
     def on_button_login_clicked(self):
-        server_url = self.lineedit_server_url.text()
-        valid_host = self.session.cmds.Flow.call(
-            self.project_oid, "update_kitsu_host", args={server_url}, kwargs={}
-        )
-
-        if not valid_host:
-            self.error_label.setText("<font color=#C71B1F>Invalid server url</font>")
-            return
-
-        # Save valid server url in login page Param for next connections
-        self.session.cmds.Flow.set_value(self.server_url_oid, server_url)
-
-        username = self.lineedit_username.text()
-        password = self.lineedit_password.text()
+        kitsu_id = self.lineedit_kitsu_id.text()
+        kitsu_password = self.lineedit_kitsu_password.text()
         logged_in = self.session.cmds.Flow.call(
             self.project_oid,
             "log_in",
             args={},
-            kwargs={"username": username, "password": password},
+            kwargs={"kitsu_id": kitsu_id, "kitsu_password": kitsu_password},
         )
 
         if not logged_in:
@@ -112,11 +93,6 @@ class LoginPage(flow.Object):
     kitsu_server_url = flow.Param("")
 
 
-class Bookmarks(flow.Object):
-
-    goto_project = flow.Parent(2)
-    user_bookmarks = flow.Child(UserBookmarks).ui(expanded=True, default_height=540)
-
 
 class SynchronizeFilesResult(flow.Action):
 
@@ -206,19 +182,52 @@ class Synchronization(flow.Object):
             )
 
 
+class LogOut(flow.Action):
+
+    ICON = ("icons.libreflow", "log_out")
+
+    def needs_dialog(self):
+        return False
+    
+    def allow_context(self, context):
+        return context and context.endswith(".details")
+    
+    def run(self, button):
+        project = self.root().project()
+        os.remove(
+            "%s/kitsu_config.json" % project.get_user_folder()
+        )
+        project.log_out()
+        
+        return self.get_result(
+            goto=project.oid(),
+            refresh=True
+        )
+
+
+class ProjectSettings(flow.Object):
+    project_nice_name = flow.Param()
+    project_thumbnail = flow.Param().ui(editor='textarea')
+
+
 class Admin(flow.Object):
 
     ICON = ("icons.gui", "team-admin")
 
+    _project = flow.Parent()
+
     current_site_name = flow.Computed(cached=True)
     root_dir = flow.Computed(cached=True)
+    kitsu_server_url = flow.Param("").watched()
 
+    project_settings = flow.Child(ProjectSettings)
     settings = flow.Child(ContextualView)  # .ui(hidden=True)
     users = flow.Child(Users)
-    user_environment = flow.Child(UserEnvironment).ui(expanded=True)
-    bookmarks = flow.Child(Bookmarks)
+    
+    user_environment = flow.Child(UserEnvironment).ui(expanded=False)
+    # bookmarks = flow.Child(Bookmarks)
 
-    default_applications = flow.Child(DefaultRunners).ui(expanded=True).injectable()
+    default_applications = flow.Child(DefaultRunners).ui(expanded=False).injectable()
     enable_filesystem_operations = flow.BoolParam(True).ui(
         label="Enable file system operations",
         tooltip="Enable operations on the OS file system (creation, move, delete) \
@@ -228,6 +237,12 @@ class Admin(flow.Object):
     process_jobs = flow.Child(Synchronize)
     login_page = flow.Child(LoginPage).ui(hidden=True)
 
+    def child_value_changed(self, child_value):
+        if child_value is self.kitsu_server_url:
+            self._project.update_kitsu_host(child_value.get())
+            self._project.kitsu_id.touch()
+            self._project.kitsu_url.touch()
+
     def compute_child_value(self, child_value):
         if child_value == self.root_dir:
             '''
@@ -255,10 +270,29 @@ class Admin(flow.Object):
                 else:
                     site_name = "default"
             child_value.set(site_name)
+        elif child_value is self.current_user_id:
+            # Check env
+            if "USER_NAME" in os.environ:
+                child_value.set(os.environ["USER_NAME"])
+                return
+            
+            # Check user file
+            user_file = os.path.join(self._project.get_user_folder(), "user.json")
+            if os.path.exists(user_file):
+                with open(user_file, "r") as f:
+                    user_config = json.load(f)
+                    child_value.set(user_config["username"])
+                    return
+
+            # Return local user name otherwise
+            child_value.set(getpass.getuser())
 
 
 class Project(KitsuProject):
 
+    log_out_action = flow.Child(LogOut).ui(label="Log out")
+
+    user = flow.Child(UserProfile)
     asset_lib = flow.Child(Assets).ui(expanded=True)
     sequences = flow.Child(Sequences).ui(expanded=True)
     admin = flow.Child(Admin)
@@ -284,19 +318,11 @@ class Project(KitsuProject):
             with open(user_file, "w+") as f:
                 user_config = dict(username=username)
                 json.dump(user_config, f)
+            
+            self.user.current_user_id.touch()
 
     def get_user(self):
-        if "USER_NAME" in os.environ:
-            return os.environ["USER_NAME"]
-        
-        user_file = os.path.join(self.get_user_folder(), "user.json")
-
-        if os.path.exists(user_file):
-            with open(user_file, "r") as f:
-                user_config = json.load(f)
-                return user_config["username"]
-
-        return getpass.getuser()
+        return self.user.current_user_id.get()
 
     def get_current_site(self):
         """
@@ -356,8 +382,9 @@ class Project(KitsuProject):
 
         return self.kitsu_api.host_is_valid()
 
-    def log_in(self, username, password):
-        success = self.kitsu_api.log_in(username, password)
+    def log_in(self, kitsu_id, kitsu_password):
+        self.update_kitsu_host(self.admin.kitsu_server_url.get())
+        success = self.kitsu_api.log_in(kitsu_id, kitsu_password)
 
         # Save authentification tokens
         if success:
@@ -372,9 +399,18 @@ class Project(KitsuProject):
                     json.dump(kitsu_config, f)
 
             # Set current user
-            self.set_user(username)
+            user_name = self.admin.users.get_user_name(kitsu_id)
+            
+            if not user_name:
+                # User profile not registered, create it
+                self.admin.users.add_user(kitsu_id)
+            
+            self.set_user(user_name)
 
         return success
+    
+    def log_out(self):
+        self.kitsu_api.log_out()
 
     def get_contextual_view(self, context_name):
         if context_name == "settings":
@@ -391,12 +427,22 @@ class Project(KitsuProject):
                 project_thumbnail="{ROOT_DIR}TECH/{PROJECT}_thumbnail.png",
             )
 
+    def get_user_bookmarks(self):
+        user = self.get_user()
+        if user in self.admin.users.mapped_names():
+            return self.admin.users[user].bookmarks
+        else:
+            return None
+
     def get_user_folder(self):
         '''
         DEPRECATED ! TO DELETE ? 
         '''
         return os.path.join(pathlib.Path.home(), ".libreflow")
 
+    def get_project_thumbnail2(self):
+        image = self.admin.project_settings.project_thumbnail.get()
+        return image
     def get_project_thumbnail(self):
         contextual_dict = get_contextual_dict(self, "settings")
         contextual_dict["ROOT_DIR"] = self.get_root()
