@@ -1,4 +1,5 @@
 import os
+import sys
 import getpass
 import time
 import datetime
@@ -8,6 +9,7 @@ import string
 import re
 import hashlib
 import timeago
+import zipfile
 
 import kabaret.app.resources as resources
 from kabaret import flow
@@ -16,8 +18,10 @@ from kabaret.subprocess_manager.flow import RunAction
 
 from .maputils import SimpleCreateAction, ClearMapAction
 
-from .site import SyncMap, Request
-from .runners import LaunchSessionWorker
+from .site import SyncMap, Request, RequestAs
+from .runners import LaunchSessionWorker, CHOICES, CHOICES_ICONS
+
+pyversion = sys.version_info
 
 class RevisionsChoiceValue(flow.values.ChoiceValue):
 
@@ -156,6 +160,7 @@ class Revision(flow.Object):
     make_current = flow.Child(MakeCurrentRevisionAction)
     create_working_copy = flow.Child(CreateWorkingCopyFromRevision)
     request = flow.Child(Request)
+    request_as = flow.Child(RequestAs)
     compute_hash_action = flow.Child(ComputeRevisionHash)
     check_hash = flow.Child(CheckRevisionHash)
 
@@ -283,7 +288,7 @@ class Revisions(flow.Map):
 
     @classmethod
     def mapped_type(cls):
-        return Revision
+        return flow.injection.injectable(Revision)
 
     def columns(self):
         columns = ["Revision", "Created by", "On"]
@@ -376,9 +381,7 @@ class History(flow.Object):
 
 
 class FileFormat(flow.values.ChoiceValue):
-
-    CHOICES = ["blend", "kra", "png", "jpg", "txt", "abc", "mov", "psd", "aep"]
-
+    CHOICES = CHOICES
 
 class CreateTrackedFileAction(flow.Action):
 
@@ -388,16 +391,7 @@ class CreateTrackedFileAction(flow.Action):
 
     file_name = flow.Param("")
     file_format = flow.Param("blend", FileFormat).ui(
-        choice_icons={
-            "blend": ("icons.libreflow", "blender"),
-            "kra": ("icons.libreflow", "krita"),
-            "png": ("icons.gui", "picture"),
-            "jpg": ("icons.gui", "picture"),
-            "txt": ("icons.flow", "notepad"),
-            "abc": ("icons.flow", "alembic"),
-            "aep": ("icons.libreflow", "afterfx"),
-            "psd": ("icons.flow", "photoshop"),
-        }
+        choice_icons=CHOICES_ICONS
     )
 
     def get_buttons(self):
@@ -885,6 +879,26 @@ class ActiveUsers(flow.Map):
         row["Revision"] = item.get()
 
 
+class RevealInExplorer(RunAction):
+
+    ICON = ('icons.flow', 'explorer')
+
+    _item = flow.Parent()
+
+    def runner_name_and_tags(self):
+        return "DefaultEditor", []
+
+    def extra_argv(self):
+        path = self._item.get_path()
+        if not os.path.isdir(path):
+            path = os.path.dirname(path)
+        
+        return [path]
+
+    def needs_dialog(self):
+        return False
+
+
 class FileSystemItem(flow.Object):
 
     _department = flow.Parent(2)
@@ -920,6 +934,7 @@ class File(FileSystemItem):
     format = flow.Param("blend", FileFormat).ui(editable=False).injectable()
     display_name = flow.Computed()
     open = flow.Child(OpenFileAction)
+    reveal = flow.Child(RevealInExplorer).ui(label="Reveal in explorer")
 
     def get_name(self):
         return "%s.%s" % (self.name(), self.format.get())
@@ -1388,20 +1403,6 @@ class ClearFileSystemMapAction(ClearMapAction):
         super(ClearFileSystemMapAction, self).run(button)
 
 
-class OpenFolderAction(RunAction):
-
-    _folder = flow.Parent()
-
-    def runner_name_and_tags(self):
-        return "DefaultEditor", []
-
-    def extra_argv(self):
-        return [os.path.normpath(self._folder.get_path())]
-
-    def needs_dialog(self):
-        return False
-
-
 class CreateFileSystemItemAction(SimpleCreateAction):
     def get_buttons(self):
         self.message.set("<h2>Create %s</h2>" % self.item_type().__name__.lower())
@@ -1438,12 +1439,210 @@ class CreateFileSystemItemAction(SimpleCreateAction):
 class Folder(FileSystemItem):
 
     _department = flow.Parent(2)
-    open = flow.Child(OpenFolderAction)
+    open = flow.Child(RevealInExplorer)
 
 
-class TrackedFolder(Folder):
-    # TODO
-    pass
+class OpenTrackedFolderRevision(RevealInExplorer):
+
+    _revision = flow.Parent()
+
+    def extra_argv(self):
+        return [os.path.normpath(self._revision.path.get())]
+
+
+class TrackedFolderRevision(Revision):
+
+    open = flow.Child(OpenTrackedFolderRevision)
+
+
+class TrackedFolderRevisions(Revisions):
+
+    @classmethod
+    def mapped_type(cls):
+        return TrackedFolderRevision
+
+
+class TrackedFolderHistory(flow.Object):
+
+    revisions = flow.Child(TrackedFolderRevisions)
+
+
+class OpenTrackedFolderAction(RevealInExplorer):
+
+    _folder = flow.Parent()
+    
+    def get_buttons(self):
+        self.message.set("You aren't seeing any revision...")
+        return ["See a revision", "Create a working copy", "Cancel"]
+
+    def needs_dialog(self):
+        seen_revision = self._folder.get_seen_revision()
+        return (
+            not seen_revision
+            or not os.path.exists(seen_revision.path.get())
+        )
+    
+    def run(self, button):
+        if button == "Cancel":
+            return
+        elif button == "See a revision":
+            return self.get_result(next_action=self._folder.see_revision)
+        elif button == "Create a working copy":
+            return self.get_result(next_action=self._folder.create_working_copy_action)
+        
+        # Else open seen revision
+        self._folder.get_seen_revision().open.run(None)
+
+
+class TrackedFolder(TrackedFile):
+
+    open = flow.Child(OpenTrackedFolderAction)
+    history = flow.Child(TrackedFolderHistory)
+    
+    def get_name(self):
+        return self.complete_name.get()
+    
+    def create_working_copy(self, reference_name=None, user_name=None):
+        if user_name is None:
+            user_name = self.root().project().get_user()
+
+        revisions = self.get_revisions()
+        working_copy = self.get_working_copy()
+
+        # Delete current working copy
+        if working_copy:
+            if os.path.exists(working_copy.path.get()):
+                shutil.rmtree(working_copy.path.get())
+            revisions.remove(working_copy.name())
+
+        # Create new one
+        working_copy = revisions.add(user_name)
+        working_copy.set_sync_status("Available")
+
+        try:
+            os.makedirs(working_copy.path.get())
+        except OSError:
+            self.root().session().log_error(
+                "Creation of working copy folder %s failed"
+                % working_copy.path.get()
+            )
+            raise
+        
+        working_copy_path = working_copy.get_path()
+
+        if reference_name:
+            # Copy reference zip to working copy folder
+            reference_path = self.get_revision(reference_name).get_path()
+            try:
+                shutil.copyfile(reference_path, working_copy_path)
+            except OSError:
+                self.root().session().log_error(
+                    "Copy of template %s failed" % reference_path
+                )
+                raise
+
+            self.root().session().log_debug(
+                "Copy current revision %s to %s" % (
+                    reference_path, working_copy_path
+                )
+            )
+
+            # Extract it
+            with zipfile.ZipFile(working_copy_path, 'r') as zip_wc:
+                zip_wc.extractall(working_copy.path.get())
+
+        return working_copy
+
+    def publish(self, comment="", keep_editing=False):
+        revisions = self.get_revisions()
+        head_revision = revisions.add()
+        head_revision.comment.set(comment)
+        head_revision.set_sync_status("Available")
+
+        working_copy = self.get_working_copy()
+        working_copy_path = working_copy.get_path()
+        head_revision_path = head_revision.get_path()
+
+        self.root().session().log_debug(
+            "Create head revision folder %s" % head_revision.path.get()
+        )
+
+        if keep_editing:
+            copy_function = shutil.copy2
+        else:
+            copy_function = shutil.move
+            revisions.remove(working_copy.name())
+        
+        # Move working copy files to head folder
+        shutil.copytree(
+            working_copy.path.get(),
+            head_revision.path.get(),
+            ignore=shutil.ignore_patterns("*.zip"),
+            copy_function=copy_function
+        )
+        if not keep_editing:
+            shutil.rmtree(working_copy.path.get())
+        
+        # Pack files in head zip file
+        files = os.listdir(head_revision.path.get())
+        with zipfile.ZipFile(head_revision_path, 'w', zipfile.ZIP_DEFLATED) as zip_head:
+            for file in files:
+                zip_head.write(
+                    os.path.join(head_revision.path.get(), file),
+                    file
+                )
+
+        revisions.touch()
+
+        # Compute published revision hash
+        head_revision.compute_hash_action.run(None)
+
+        return head_revision
+
+    def make_current(self, revision):
+        revision_path = os.path.join(revision.path.get(), revision.file_name.get())
+        current_path, current_name = self.get_current_revision_params()
+        current_revision_path = os.path.join(current_path, current_name)
+
+        if not os.path.exists(current_path):
+            os.makedirs(current_path)
+        
+        if self.format.get() == "zip":
+            # Copy files to current folder
+
+            if pyversion.minor < 8:
+                # DIRTYYYYYYY TRICK for python 3.7
+                shutil.rmtree(current_path)
+                shutil.copytree(
+                    revision.path.get(),
+                    current_path,
+                    ignore=shutil.ignore_patterns("*.zip"),
+                )
+            else:
+                shutil.copytree(
+                    revision.path.get(),
+                    current_path,
+                    ignore=shutil.ignore_patterns("*.zip"),
+                    dirs_exist_ok=True
+                )
+            # Pack files in head zip file
+            files = os.listdir(current_path)
+            with zipfile.ZipFile(current_revision_path, 'w', zipfile.ZIP_DEFLATED) as zip_current:
+                for file in files:
+                    zip_current.write(
+                        os.path.join(current_path, file),
+                        file
+                    )
+            # shutil.make_archive(current_revision_path, 'zip')
+
+        self.current_revision.set(revision.name())
+        self.get_revisions().touch()
+
+    def compute_child_value(self, child_value):
+        if child_value is self.display_name:
+            child_value.set(self.name())
+        else:
+            TrackedFile.compute_child_value(self, child_value)
 
 
 class CreateFileAction(CreateFileSystemItemAction):
@@ -1467,16 +1666,55 @@ class CreateFolderAction(CreateFileSystemItemAction):
         return self._map.add_folder(name)
 
 
+class CreateTrackedFolderAction(flow.Action):
+
+    ICON = ("icons.gui", "plus-sign-in-a-black-circle")
+
+    _files = flow.Parent()
+
+    folder_name = flow.Param("")
+
+    def get_buttons(self):
+        self.message.set("<h3>Create folder</h3>")
+        return ["Create", "Cancel"]
+
+    def run(self, button):
+        if button == "Cancel":
+            return
+
+        settings = get_contextual_dict(self, "settings")
+        file_category = settings.get("file_category", None)
+
+        name = self.folder_name.get()
+        prefix = ""
+
+        if file_category is not None:
+            if file_category == "PROD":
+                prefix = "{file_category}_{sequence}_{shot}_{department}_"
+            elif file_category == "LIB":
+                prefix = "{file_category}_{asset_name}_{department}_"
+
+            prefix = prefix.format(**settings)
+
+        self.root().session().log_debug(
+            "Creating folder %s" % name
+        )
+
+        self._files.add_tracked_folder(name, prefix + name)
+        self._files.touch()
+
+
 class FileSystemMap(flow.Map):
 
     create_file = flow.Child(CreateTrackedFileAction).injectable()
+    create_folder = flow.Child(CreateTrackedFolderAction).injectable()
     create_untracked_folder = flow.Child(CreateFolderAction)
     create_untracked_file = flow.Child(CreateFileAction)
     clear_map = flow.Child(ClearFileSystemMapAction).ui(hidden=True)
 
     @classmethod
     def mapped_type(cls):
-        return FileSystemItem
+        return flow.injection.injectable(FileSystemItem)
 
     def columns(self):
         return ["Lock", "Name", "Last comment"]
@@ -1599,7 +1837,7 @@ class FileSystemMap(flow.Map):
 
     def add_tracked_file(self, name, extension, complete_name):
         key = "%s_%s" % (name, extension)
-        file = self.add(key, object_type=TrackedFile)
+        file = self.add(key, object_type=flow.injection.injectable(TrackedFile))
         file.format.set(extension)
         file.complete_name.set(complete_name)
 
@@ -1637,7 +1875,42 @@ class FileSystemMap(flow.Map):
         return file
 
     def add_tracked_folder(self, name, complete_name):
-        raise NotImplementedError("Not implemented yet")
+        folder = self.add(name, object_type=TrackedFolder)
+        folder.format.set("zip")
+        folder.complete_name.set(complete_name)
+
+        if self.root().project().admin.enable_filesystem_operations.get():
+            # Create file folder
+            try:
+                self.root().session().log_debug(
+                    "Create tracked folder '{}'".format(folder.get_path())
+                )
+                os.makedirs(folder.get_path())
+            except OSError:
+                self.root().session().log_error(
+                    "Creation of tracked folder '{}' failed.".format(folder.get_path())
+                )
+                pass
+
+            # Create current revision folder
+            current_revision_folder = os.path.join(folder.get_path(), "current")
+
+            try:
+                self.root().session().log_debug(
+                    "Create current revision folder '{}'".format(
+                        current_revision_folder
+                    )
+                )
+                os.mkdir(current_revision_folder)
+            except OSError:
+                self.root().session().log_error(
+                    "Creation of current revision folder '{}' failed".format(
+                        current_revision_folder
+                    )
+                )
+                pass
+
+        return folder
 
     def clear(self):
         if self.root().project().admin.enable_filesystem_operations.get():
