@@ -71,10 +71,7 @@ class Shot(KitsuShot):
 
     def get_default_contextual_edits(self, context_name):
         if context_name == "settings":
-            default_settings = get_contextual_dict(self._sequence, context_name)
-            path = default_settings.get("path", "UNKNOWN_PATH")
-
-            return dict(path=os.path.join(path, self.name()), shot=self.name())
+            return dict(shot=self.name())
 
 
 class Shots(ItemMap):
@@ -132,10 +129,7 @@ class Sequence(KitsuSequence):
 
     def get_default_contextual_edits(self, context_name):
         if context_name == "settings":
-            default_settings = get_contextual_dict(self._map, context_name)
-            path = default_settings.get("path", "UNKNOWN_PATH")
-
-            return dict(path=os.path.join(path, self.name()), sequence=self.name())
+            return dict(sequence=self.name())
 
 
 class ClearSequencesAction(ClearMapAction):
@@ -172,4 +166,4 @@ class Sequences(ItemMap):
 
     def get_default_contextual_edits(self, context_name):
         if context_name == "settings":
-            return dict(path="PROD", file_category="PROD")
+            return dict(file_category="PROD")
