@@ -32,6 +32,9 @@ class KitsuAPIWrapper(flow.Object):
             return False
 
         return True
+    
+    def log_out(self):
+        gazu.log_out()
 
     def get_tokens(self):
         return gazu.client.default_client.tokens
@@ -158,10 +161,9 @@ class KitsuObject(flow.Object):
     Subclasses must implement the *kitsu_dict* and *compute_child_value* methods.
     """
 
-    with flow.group("Kitsu"):
-        kitsu_settings = flow.Child(KitsuSettings)
-        kitsu_url = flow.Computed(computed_value_type=Url)
-        kitsu_id = flow.Param().ui(editable=False)
+    kitsu_settings = flow.Child(KitsuSettings).ui(hidden=True)
+    kitsu_url = flow.Computed(computed_value_type=Url).ui(hidden=True)
+    kitsu_id = flow.Param().ui(editable=False).ui(hidden=True)
 
     def kitsu_setting_names(self):
         """
@@ -312,13 +314,12 @@ class SyncFromKitsu(flow.Action):
 
 class KitsuProject(KitsuObject):
 
-    with flow.group("Kitsu"):
-        kitsu_name = flow.Param("").watched()
-        kitsu_url = flow.Computed()
-        kitsu_id = flow.Computed()
+    kitsu_name = flow.Param("").watched().ui(hidden=True)
+    kitsu_url = flow.Computed().ui(hidden=True)
+    kitsu_id = flow.Computed().ui(hidden=True)
 
-        kitsu_api = flow.Child(KitsuAPIWrapper).ui(hidden=True)
-        sync_from_kitsu = flow.Child(SyncFromKitsu).injectable().ui(label="Synchronize")
+    kitsu_api = flow.Child(KitsuAPIWrapper).ui(hidden=True)
+    sync_from_kitsu = flow.Child(SyncFromKitsu).injectable().ui(label="Synchronize", hidden=True)
 
     def kitsu_dict(self):
         project_name = self.kitsu_name.get()
