@@ -10,6 +10,22 @@ from kabaret.subprocess_manager.runner_factory import Runner, logger
 from kabaret.subprocess_manager.flow import RunAction
 
 
+CHOICES = ["blend", "kra", "png", "jpg", "txt", "abc", "mov", "psd", "aep", "zip"]
+CHOICES_ICONS = {
+            "blend": ("icons.libreflow", "blender"),
+            "kra": ("icons.libreflow", "krita"),
+            "png": ("icons.gui", "picture"),
+            "jpg": ("icons.gui", "picture"),
+            "txt": ("icons.flow", "notepad"),
+            "abc": ("icons.flow", "alembic"),
+            "aep": ("icons.libreflow", "afterfx"),
+            "psd": ("icons.flow", "photoshop"),
+            "mov": ("icons.flow", "quicktime"),
+            "zip": ("icons.flow", "file"),
+        }
+
+
+
 # Runners
 # -----------------
 
@@ -78,7 +94,6 @@ class DefaultEditor(BaseRunner):
             return "open"
         elif platform.system() == "Linux":
             return "xdg-open"
-
         return None
 
     def run(self):
@@ -352,7 +367,7 @@ class DefaultRunners(flow.DynamicMap):
         return DefaultRunner
 
     def mapped_names(self, page_num=0, page_size=None):
-        return ["blend", "kra", "png", "jpg", "txt", "abc", "mov", "psd", "aep"]
+        return CHOICES
 
     def columns(self):
         return ["Extension", "Default application", "Version"]
