@@ -333,16 +333,18 @@ class TrackedFile(baseflow.file.TrackedFile):
         return False
 
     def get_playblast_folder(self):
-        settings = get_contextual_dict(
-            self._department, "settings", ["path"]
+        return os.path.join(
+            self.root().project().get_root(),
+            self._department.path.get(),
+            "preview"
         )
-        return os.path.join(self.root().project().get_root(), settings["path"], "preview")
     
     def get_render_path(self):
-        settings = get_contextual_dict(
-            self._department, "settings", ["path"]
+        return os.path.join(
+            self.root().project().get_root(),
+            self._department.path.get(),
+            "render"
         )
-        return os.path.join(self.root().project().get_root(), settings["path"], "render")
 
 
 class FileSystemMap(baseflow.file.FileSystemMap):
